@@ -10,7 +10,7 @@ use Scalar::Util qw/blessed refaddr reftype/;
 use overload ();
 use Symbol ();
 
-our $VERSION = "0.08";
+our $VERSION = "0.09";
 
 sub visit {
 	my ( $self, $data ) = @_;
@@ -77,22 +77,48 @@ sub visit_hash {
 	my ( $self, $hash ) = @_;
 
 	if ( not defined wantarray ) {
-		$self->visit( $_ ) for ( values %$hash );
+		foreach my $key ( keys %$hash ) {
+			$self->visit_hash_entry( $key, $hash->{$key}, $hash );
+		}
 	} else {
-		return $self->retain_magic( $hash, { map { $_ => $self->visit( $hash->{$_} ) } keys %$hash } );
+		return $self->retain_magic( $hash, { map { $self->visit_hash_entry( $_, $hash->{$_}, $hash ) } keys %$hash } );
 	}
 }
 
+sub visit_hash_entry {
+	my ( $self, $key, $value, $hash ) = @_;
+
+	return (
+		$self->visit_hash_key($key,$value,$hash),
+		$self->visit_hash_value($_[2],$key,$hash) # retain aliasing semantics
+	);
+}
+
+sub visit_hash_key {
+	my ( $self, $key, $value, $hash ) = @_;
+	$self->visit($key);
+}
+
+sub visit_hash_value {
+	my ( $self, $value, $key, $hash ) = @_;
+	$self->visit($_[1]); # retain it's aliasing semantics
+}
+
 sub visit_array {
 	my ( $self, $array ) = @_;
 
 	if ( not defined wantarray ) {
-		$self->visit( $_ ) for @$array;	
+		$self->visit_array_entry( $array->[$_], $_, $array ) for 0 .. $#$array
 	} else {
-		return $self->retain_magic( $array, [ map { $self->visit( $_ ) } @$array ] );
+		return $self->retain_magic( $array, [ map { $self->visit_array_entry( $array->[$_], $_, $array ) } 0 .. $#$array ] );
 	}
 }
 
+sub visit_array_entry {
+	my ( $self, $value, $index, $array ) = @_;
+	$self->visit($_[1]);
+}
+
 sub visit_scalar {
 	my ( $self, $scalar ) = @_;
 	return $self->retain_magic( $scalar, \$self->visit( $$scalar ) );
@@ -182,6 +208,16 @@ various other visiting methods, based on the data's type.
 If the value is a blessed object, C<visit> calls this method. The base
 implementation will just forward to C<visit_value>.
 
+=item visit_ref $value
+
+Generic recursive visitor. All non blessed values are given to this.
+
+C<visit_object> can delegate to this method in order to visit the object
+anyway.
+
+This will check if the visitor can handle C<visit_$reftype> (lowercase), and if
+not delegate to C<visit_value> instead.
+
 =item visit_array $array_ref
 
 =item visit_hash $hash_ref
@@ -197,6 +233,24 @@ These methods are called for the corresponding container type.
 If the value is anything else, this method is called. The base implementation
 will return $value.
 
+=item visit_hash_entry $key, $value, $hash
+
+Delegates to C<visit_hash_key> and C<visit_hash_value>. The value is passed as
+C<$_[2]> so that it is aliased.
+
+=item visit_hash_key $key, $value, $hash
+
+Calls C<visit> on the key and returns it.
+
+=item visit_hash_value $value, $key, $hash
+
+The value will be aliased (passed as C<$_[1]>).
+
+=item visit_array_entry $value, $index, $array
+
+Delegates to C<visit> on value. The value is passed as C<$_[1]> to retain
+aliasing.
+
 =back
 
 =head1 RETURN VALUE
@@ -219,8 +273,23 @@ C<visit_hash>.
 
 =head1 TODO
 
+=over 4
+
+=item *
+
 Add support for "natural" visiting of trees.
 
+=item *
+
+Expand C<retain_magic> to support tying at the very least, or even more with
+L<Variable::Magic> if possible.
+
+Tied values might be redirected to an alternate handler that builds a new empty
+value, and ties it to a visited clone of the object the original is tied to
+using a trampoline class. Look into this.
+
+=back
+
 =head1 SEE ALSO
 
 L<Tree::Simple::VisitorFactory>, L<Data::Traverse>
